@@ -3,7 +3,7 @@
 
 package Net::SNMP::Transport;
 
-# $Id: Transport.pm,v 1.2 2005/07/20 13:53:07 dtown Exp $
+# $Id: Transport.pm,v 2.0 2005/10/20 14:14:47 dtown Rel $
 
 # Base object for the Net::SNMP Transport Domain objects.
 
@@ -19,7 +19,7 @@ use strict;
 
 ## Version of the Net::SNMP::Transport module
 
-our $VERSION = v1.0.2;
+our $VERSION = v2.0.0;
 
 ## Handle importing/exporting of symbols
 
@@ -30,23 +30,17 @@ our @ISA = qw( Exporter );
 our @EXPORT_OK = qw( TRUE FALSE );
 
 our %EXPORT_TAGS = (
-   domains => [ 
-      qw( DOMAIN_UDP DOMAIN_UDPIPV6 DOMAIN_TCPIPV4 DOMAIN_TCPIPV6 ) 
+   domains => [
+      qw( DOMAIN_UDP DOMAIN_UDPIPV4 DOMAIN_UDPIPV6 DOMAIN_UDPIPV6Z
+          DOMAIN_TCPIPV4 DOMAIN_TCPIPV6 DOMAIN_TCPIPV6Z )  
    ], 
    msgsize => [ qw( MSG_SIZE_DEFAULT MSG_SIZE_MINIMUM MSG_SIZE_MAXIMUM ) ], 
    ports   => [ qw( SNMP_PORT SNMP_TRAP_PORT )                           ],
    retries => [ qw( RETRIES_DEFAULT RETRIES_MINIMUM RETRIES_MAXIMUM )    ],
-   timeout => [ qw( TIMEOUT_DEFAULT TIMEOUT_MINIMUM TIMEOUT_MAXIMUM )    ],
-   _array  => [ 
-      qw( _DSTNAME _MAXSIZE _RETRIES _TIMEOUT _SOCKET _SRCADDR _DSTADDR
-          _ERROR _REASM_BUFFER _REASM_LENGTH )
-   ],
-   _shared => [ qw( _SHARED_SOCKET _SHARED_REFC _SHARED_MAXSIZE )        ]
+   timeout => [ qw( TIMEOUT_DEFAULT TIMEOUT_MINIMUM TIMEOUT_MAXIMUM )    ]
 );
 
-Exporter::export_ok_tags(
-   qw( domains msgsize ports retries timeout _array _shared ) 
-);
+Exporter::export_ok_tags( qw( domains msgsize ports retries timeout ) );
 
 $EXPORT_TAGS{ALL} = [ @EXPORT_OK ];
 
@@ -55,14 +49,17 @@ $EXPORT_TAGS{ALL} = [ @EXPORT_OK ];
 # RFC 3417 Transport Mappings for SNMP
 # Presuhn, Case, McCloghrie, Rose, and Waldbusser; December 2002
 
-sub DOMAIN_UDP()           { '1.3.6.1.6.1.1' } # snmpUDPDomain
+sub DOMAIN_UDP()            { '1.3.6.1.6.1.1' }  # snmpUDPDomain
 
 # RFC 3419 Textual Conventions for Transport Addresses
 # Consultant, Schoenwaelder, and Braunschweig; December 2002
 
-sub DOMAIN_UDPIPV6() { '1.3.6.1.2.1.100.1.2' } # transportDomainUdpIpv6
-sub DOMAIN_TCPIPV4() { '1.3.6.1.2.1.100.1.5' } # transportDomainTcpIpv4
-sub DOMAIN_TCPIPV6() { '1.3.6.1.2.1.100.1.6' } # transportDomainTcpIpv6
+sub DOMAIN_UDPIPV4()  { '1.3.6.1.2.1.100.1.1' }  # transportDomainUdpIpv4
+sub DOMAIN_UDPIPV6()  { '1.3.6.1.2.1.100.1.2' }  # transportDomainUdpIpv6
+sub DOMAIN_UDPIPV6Z() { '1.3.6.1.2.1.100.1.4' }  # transportDomainUdpIpv6z
+sub DOMAIN_TCPIPV4()  { '1.3.6.1.2.1.100.1.5' }  # transportDomainTcpIpv4
+sub DOMAIN_TCPIPV6()  { '1.3.6.1.2.1.100.1.6' }  # transportDomainTcpIpv6
+sub DOMAIN_TCPIPV6Z() { '1.3.6.1.2.1.100.1.8' }  # transportDomainTcpIpv6z
 
 ## SNMP well-known ports
 
@@ -73,7 +70,7 @@ sub SNMP_TRAP_PORT()         { 162 }
 
 sub MSG_SIZE_DEFAULT()      {  484 }  
 sub MSG_SIZE_MINIMUM()      {  484 }
-sub MSG_SIZE_MAXIMUM()     { 65535 }  # 2147483647 is not reasonable
+sub MSG_SIZE_MAXIMUM()     { 65535 }   # 2147483647 is not reasonable
 
 sub RETRIES_DEFAULT()         {  1 }
 sub RETRIES_MINIMUM()         {  0 }
@@ -88,19 +85,6 @@ sub TIMEOUT_MAXIMUM()       { 60.0 }
 sub TRUE()                  { 0x01 }
 sub FALSE()                 { 0x00 }
 
-## Object array indexes
-
-sub _DSTNAME()                 { 0 }   # Destination hostname
-sub _MAXSIZE()                 { 1 }   # maxMsgSize
-sub _RETRIES()                 { 2 }   # Number of retransmissions
-sub _TIMEOUT()                 { 3 }   # Timeout period in seconds
-sub _SOCKET()                  { 4 }   # Socket object
-sub _SRCADDR()                 { 5 }   # Source sockaddr
-sub _DSTADDR()                 { 6 }   # Destination sockaddr
-sub _ERROR()                   { 7 }   # Error message
-sub _REASM_BUFFER()            { 8 }   # Reassembly buffer for data
-sub _REASM_LENGTH()            { 9 }   # Expected length of reassembled data
-
 ## Shared socket array indexes
 
 sub _SHARED_SOCKET()           { 0 }   # Shared Socket object
@@ -125,7 +109,7 @@ sub new
 {
    my ($class, %argv) = @_;
 
-   my $domain = DOMAIN_UDP;
+   my $domain = DOMAIN_UDPIPV4;
    my $error  = '';
 
    # See if a Transport Layer Domain argument has been passed.
@@ -136,16 +120,18 @@ sub new
 
          # Allow the user some flexability
          my $supported = {
-            'udp4',          DOMAIN_UDP,
-            'udpip4',        DOMAIN_UDP,
-            'udpipv4',       DOMAIN_UDP,
-            'udp/ipv4',      DOMAIN_UDP,
-            DOMAIN_UDP,      DOMAIN_UDP,
+            'udp4',          DOMAIN_UDPIPV4,
+            'udpip4',        DOMAIN_UDPIPV4,
+            'udpipv4',       DOMAIN_UDPIPV4,
+            'udp/ipv4',      DOMAIN_UDPIPV4,
+            DOMAIN_UDP,      DOMAIN_UDPIPV4,
+            DOMAIN_UDPIPV4,  DOMAIN_UDPIPV4,
             'udp6',          DOMAIN_UDPIPV6,
             'udpip6',        DOMAIN_UDPIPV6,
             'udpipv6',       DOMAIN_UDPIPV6,
             'udp/ipv6',      DOMAIN_UDPIPV6,
             DOMAIN_UDPIPV6,  DOMAIN_UDPIPV6,
+            DOMAIN_UDPIPV6Z, DOMAIN_UDPIPV6,
             'tcp4',          DOMAIN_TCPIPV4,
             'tcpip4',        DOMAIN_TCPIPV4,
             'tcpipv4',       DOMAIN_TCPIPV4,
@@ -156,6 +142,7 @@ sub new
             'tcpipv6',       DOMAIN_TCPIPV6,
             'tcp/ipv6',      DOMAIN_TCPIPV6,
             DOMAIN_TCPIPV6,  DOMAIN_TCPIPV6,
+            DOMAIN_TCPIPV6Z, DOMAIN_TCPIPV6
          };
 
          my $key   = $_;
@@ -221,7 +208,7 @@ sub max_msg_size
 
       if ($size =~ /^\d+$/) {
          if (($size >= MSG_SIZE_MINIMUM) && ($size <= MSG_SIZE_MAXIMUM)) { 
-            $this->_shared_maxsize($this->[_MAXSIZE] = $size);
+            $this->_shared_max_size($this->{_max_msg_size} = $size);
          } else {
             return $this->_error(
                'Invalid maxMsgSize value [%s], range %d - %d octets',
@@ -234,7 +221,7 @@ sub max_msg_size
 
    }
 
-   $this->[_MAXSIZE];
+   $this->{_max_msg_size};
 }
 
 sub timeout
@@ -247,7 +234,7 @@ sub timeout
 
       if ($timeout =~ /^\d+(\.\d+)?$/) {
          if (($timeout >= TIMEOUT_MINIMUM) && ($timeout <= TIMEOUT_MAXIMUM)) {
-            $this->[_TIMEOUT] = $timeout;
+            $this->{_timeout} = $timeout;
          } else {
             return $this->_error(
                'Invalid timeout value [%s], range %03.01f - %03.01f seconds',
@@ -260,7 +247,7 @@ sub timeout
 
    }
 
-   $this->[_TIMEOUT];
+   $this->{_timeout};
 }
 
 sub retries
@@ -273,7 +260,7 @@ sub retries
 
       if ($retries =~ /^\d+$/) {
          if (($retries >= RETRIES_MINIMUM) && ($retries <= RETRIES_MAXIMUM)) {
-            $this->[_RETRIES] = $retries;
+            $this->{_retries} = $retries;
          } else {
             return $this->_error(
                'Invalid retries value [%s], range %d - %d',
@@ -286,7 +273,12 @@ sub retries
 
    }
 
-   $this->[_RETRIES];
+   $this->{_retries};
+}
+
+sub agent_addr
+{
+   '0.0.0.0';
 }
 
 sub connectionless
@@ -294,115 +286,172 @@ sub connectionless
    TRUE;  
 }
 
-sub domain
-{
-   '0.0';
-}
-
-sub name
-{
-   '<unknown>';
-}
-
-sub srcaddr
-{
-   $_[0]->[_SOCKET]->sockaddr || $_[0]->_addr_any;
-}
-
-sub srcport
-{
-   $_[0]->[_SOCKET]->sockport || 0;
-}
-
-sub srchost
-{
-   $_[0]->[_SOCKET]->sockhost || $_[0]->_addr_ntoa($_[0]->srcaddr);
-}
-
-sub srcname
-{
-   my ($this) = @_;
-
-   my $srcaddr = $this->srcaddr;
-
-   if ($srcaddr eq $this->_addr_any) {
-      eval {
-         require Sys::Hostname;
-         $srcaddr = $this->_addr_aton(Sys::Hostname::hostname());
-      };
-      $srcaddr = $this->_addr_any if (!defined($srcaddr) || $@);
-   }
-
-   $this->_addr_ntoa($srcaddr);
-}
-
-sub dstaddr
-{
-   ($_[0]->_addr_pack($_[0]->[_DSTADDR]))[1];
-}
-
-sub dstport
-{
-   ($_[0]->_addr_pack($_[0]->[_DSTADDR]))[0];
-}
-
-sub dsthost
-{
-   $_[0]->_addr_ntoa($_[0]->dstaddr);
-}
-
-sub dstname
-{
-   $_[0]->[_DSTNAME];
-}
-
-sub recvaddr
-{
-   $_[0]->[_SOCKET]->peeraddr || $_[0]->_addr_any;
-}
-
-sub recvport
-{
-   $_[0]->[_SOCKET]->peerport || 0;
-}
-
-sub recvhost
-{
-   $_[0]->[_SOCKET]->peerhost || $_[0]->_addr_ntoa($_[0]->recvaddr);
-}
-
-sub socket
-{
-   $_[0]->[_SOCKET];
-}
-
-sub fileno
-{
-   $_[0]->[_SOCKET]->fileno;
-}
-
-sub error
-{
-   $_[0]->[_ERROR] || '';
-}
-
 sub debug
 {
    (@_ == 2) ? $DEBUG = ($_[1]) ? TRUE : FALSE : $DEBUG;
 }
 
+sub domain
+{
+   '0.0';
+}
+
+sub error
+{
+   $_[0]->{_error} || '';
+}
+
+sub fileno
+{
+   defined($_[0]->{_socket}) ? $_[0]->{_socket}->fileno : undef;
+}
+
+sub socket
+{
+   $_[0]->{_socket};
+}
+
+sub type
+{
+   '<unknown>'; # unknown(0)
+}
+
+sub sock_name
+{
+   $_[0]->{_sock_name};
+}
+
+sub sock_hostname
+{
+   $_[0]->{_sock_hostname} || $_[0]->sock_address;
+}
+
+sub sock_address
+{
+   $_[0]->_address($_[0]->sock_name);
+}
+
+sub sock_addr
+{
+   $_[0]->_addr($_[0]->sock_name);
+}
+
+sub sock_port
+{
+   $_[0]->_port($_[0]->sock_name);
+}
+
+sub sock_taddress
+{
+   $_[0]->_taddress($_[0]->sock_name);
+}
+
+sub sock_taddr
+{
+   $_[0]->_taddr($_[0]->sock_name);
+}
+
+sub sock_tdomain
+{
+   $_[0]->_tdomain($_[0]->sock_name);
+}
+
+sub dest_name
+{
+   $_[0]->{_dest_name};
+}
+
+sub dest_hostname
+{
+   $_[0]->{_dest_hostname} || $_[0]->dest_address;
+}
+
+sub dest_address
+{
+   $_[0]->_address($_[0]->dest_name);
+}
+
+sub dest_addr
+{
+   $_[0]->_addr($_[0]->dest_name);
+}
+
+sub dest_port
+{
+   $_[0]->_port($_[0]->dest_name);
+}
+
+sub dest_taddress
+{
+   $_[0]->_taddress($_[0]->dest_name);
+}
+
+sub dest_taddr
+{
+   $_[0]->_taddr($_[0]->dest_name);
+}
+
+sub dest_tdomain
+{
+   $_[0]->_tdomain($_[0]->dest_name);
+}
+
+sub peer_name
+{
+   $_[0]->{_socket}->peername || $_[0]->dest_name;
+}
+
+sub peer_hostname
+{
+   $_[0]->peer_address;
+}
+
+sub peer_address
+{
+   $_[0]->_address($_[0]->peer_name);
+}
+
+sub peer_addr
+{
+   $_[0]->_addr($_[0]->peer_name);
+}
+
+sub peer_port
+{
+   $_[0]->_port($_[0]->peer_name);
+}
+
+sub peer_taddress
+{
+   $_[0]->_taddress($_[0]->peer_name);
+}
+
+sub peer_taddr
+{
+   $_[0]->_taddr($_[0]->peer_name);
+}
+
+sub peer_tdomain
+{
+   $_[0]->_tdomain($_[0]->peer_name);
+}
+
 sub AUTOLOAD
 {
-   my ($this) = @_;
+   my $this = shift;
 
    return if $AUTOLOAD =~ /::DESTROY$/;
 
    $AUTOLOAD =~ s/.*://;
 
    if (ref($this)) {
-      $this->_error(
-         'Feature not supported by this Transport Domain [%s]', $AUTOLOAD
-      );
+      if (defined($this->{_socket}) && ($this->{_socket}->can($AUTOLOAD))) {
+         $this->{_socket}->$AUTOLOAD(@_);
+      } else { 
+         $this->_error(
+            'Feature not supported by this Transport Domain [%s]', $AUTOLOAD
+         );
+      }
    } else {
       die sprintf('Unsupported function call [%s]', $AUTOLOAD);
    }
@@ -418,165 +467,217 @@ sub DESTROY
    # Decrement the reference count and clear the shared
    # socket structure if no one is using it.
 
-   return unless (defined($this->[_SRCADDR]) &&
-                    exists($SOCKETS->{$this->[_SRCADDR]}));
+   return unless (defined($this->{_sock_name}) &&
+                  exists($SOCKETS->{$this->{_sock_name}}));
 
-   if (--$SOCKETS->{$this->[_SRCADDR]}->[_SHARED_REFC] < 1) {
-      delete($SOCKETS->{$this->[_SRCADDR]});
+   if (--$SOCKETS->{$this->{_sock_name}}->[_SHARED_REFC] < 1) {
+      delete($SOCKETS->{$this->{_sock_name}});
    }
 }
 
+## Deprecated accessor methods
+
+sub DEPRECATED
+{
+   my ($this, $method) = splice(@_, 0, 2);
+
+   warn
+      sprintf(
+         "%s() is deprecated, use %s() instead at %s line %d.\n",
+         (caller(1))[3], $method, (caller(1))[1], (caller(1))[2]
+      );
+
+   $this->${\$method}(@_);
+}
+
+sub name     { $_[0]->DEPRECATED('type');          }
+
+sub srcaddr  { $_[0]->DEPRECATED('sock_addr');     }
+
+sub srcport  { $_[0]->DEPRECATED('sock_port');     }
+
+sub srchost  { $_[0]->DEPRECATED('sock_address');  }
+
+sub srcname  { $_[0]->DEPRECATED('sock_address');  }
+
+sub dstaddr  { $_[0]->DEPRECATED('dest_addr');     }
+
+sub dstport  { $_[0]->DEPRECATED('dest_port');     }
+
+sub dsthost  { $_[0]->DEPRECATED('dest_address');  }
+
+sub dstname  { $_[0]->DEPRECATED('dest_hostname'); }
+
+sub recvaddr { $_[0]->DEPRECATED('peer_addr');     }
+
+sub recvport { $_[0]->DEPRECATED('peer_port');     }
+
+sub recvhost { $_[0]->DEPRECATED('peer_address');  }
+
+
 # [private methods] ----------------------------------------------------------
 
 sub _new
 {
    my ($class, %argv) = @_;
 
-   my $this = bless [
-      'localhost',                # Destination hostname
-      $class->_msg_size_default,  # maxMsgSize
-      RETRIES_DEFAULT,            # Number of retransmissions
-      TIMEOUT_DEFAULT,            # Timeout period in seconds
-      undef,                      # Socket object
-      undef,                      # Source sockaddr
-      undef,                      # Destination sockaddr
-      undef,                      # Error message
-      '',                         # Reassembly buffer
-      0                           # Reassembly length
-   ], $class;
+   my $this = bless {
+      '_dest_hostname' => 'localhost',                # Destination hostname
+      '_dest_name'     => undef,                      # Destination sockaddr
+      '_error'         => undef,                      # Error message
+      '_max_msg_size'  => $class->_msg_size_default,  # maxMsgSize
+      '_retries'       => RETRIES_DEFAULT,            # Number of retries      
+      '_socket'        => undef,                      # Socket object
+      '_sock_hostname' => '',                         # Socket hostname
+      '_sock_name'     => undef,                      # Socket sockaddr
+      '_timeout'       => TIMEOUT_DEFAULT,            # Timeout period (secs)
+   }, $class;
 
-   my $src_addr = $this->_addr_any;
-   my $src_port = 0;
-   my $dst_addr = $this->_addr_loopback;
-   my $dst_port = SNMP_PORT;
-   my $listen   = 0;
+   # Default the values for the "name (sockaddr) hashes".
 
-   # Validate the passed arguments
+   my $sock_nh = { port => 0,         addr => $this->_addr_any      };
+   my $dest_nh = { port => SNMP_PORT, addr => $this->_addr_loopback };
+
+   # Validate the "port" arguments first to allow for a consistency
+   # check with any values passed with the "address" arguments.
+
+   my ($dest_port, $sock_port, $listen) = (undef, undef, 0); 
 
    foreach (keys %argv) {
 
       if (/^-?debug$/i) {
-         $this->debug($argv{$_});
-      } elsif ((/^-?dstaddr$/i) || (/^-?hostname$/i)) {
-         $this->[_DSTNAME] = $argv{$_};
-         if (!defined($dst_addr = $this->_addr_aton($argv{$_}))) {
-            $this->_error(
-               "Unable to resolve destination %s address '%s'",
-               $this->name, $argv{$_}
-            );
-         }
-      } elsif ((/^-?dstport$/i) || (/^-?port$/i)) {
-         if (!defined($dst_port = $this->_serv_aton($argv{$_}))) {
-            $this->_error(
-               "Unable to resolve destination %s service '%s'",
-               $this->name, $argv{$_}
-            );
-         }
-      } elsif ((/^-?srcaddr$/i) || (/^-?localaddr$/i)) {
-         if (!defined($src_addr = $this->_addr_aton($argv{$_}))) {
-            $this->_error(
-               "Unable to resolve local %s address '%s'", $this->name, $argv{$_}
-            );
-         }
-      } elsif ((/^-?srcport$/i) || (/^-?localport$/i)) {
-         if (!defined($src_port = $this->_serv_aton($argv{$_}))) {
-            $this->_error(
-               "Unable to resolve local %s service '%s'", $this->name, $argv{$_}
-            );
-         }
-      } elsif (/^-?domain$/i) {
+         $this->debug(delete($argv{$_}));
+      } elsif (/^-?(?:de?st|peer)?port$/i) {
+         $this->_service_resolve(delete($argv{$_}), $dest_nh);
+         $dest_port = $dest_nh->{port};
+      } elsif (/^-?(?:src|sock|local)port$/i) {
+         $this->_service_resolve(delete($argv{$_}), $sock_nh);
+         $sock_port = $sock_nh->{port};
+      }
+
+      if (defined($this->{_error})) {
+         return wantarray ? (undef, $this->{_error}) : undef;
+      }
+   }
+
+   # Validate the rest of the arguments.
+
+   foreach (keys %argv) {
+
+      if (/^-?domain$/i) {
          if ($argv{$_} ne $this->domain) {
             $this->_error('Invalid Transport Domain [%s]', $argv{$_});
          }
+      } elsif ((/^-?hostname$/i) || (/^-?(?:de?st|peer)?addr$/i)) {
+         $this->_hostname_resolve($this->{_dest_hostname} = $argv{$_}, $dest_nh);
+         if (defined($dest_port) && ($dest_port != $dest_nh->{port})) {
+            $this->_error(
+               'Inconsistent %s port information specified [%d != %d]',
+               $this->type, $dest_port, $dest_nh->{port}
+            );
+         }
+      } elsif (/^-?(?:src|sock|local)addr$/i) {
+         $this->_hostname_resolve($this->{_sock_hostname} = $argv{$_}, $sock_nh);
+         if (defined($sock_port) && ($sock_port != $sock_nh->{port})) {
+            $this->_error(
+               'Inconsistent %s port information specified [%d != %d]',
+               $this->type, $sock_port, $sock_nh->{port}
+            );
+         }
+      } elsif (/^-?listen$/i) {
+         if (($argv{$_} !~ /^\d+$/) || ($argv{$_} < 1)) {
+            $this->_error('Expected positive non-zero listen queue size');
+         } elsif (!$this->connectionless) {
+            $listen = $argv{$_};
+         }
       } elsif ((/^-?maxmsgsize$/i) || (/^-?mtu$/i)) {
          $this->max_msg_size($argv{$_});
       } elsif (/^-?retries$/i) {
          $this->retries($argv{$_});
       } elsif (/^-?timeout$/i) {
          $this->timeout($argv{$_});
-      } elsif (/^-?listen$/i) {
-         if (($argv{$_} !~ /^\d+$/) || ($argv{$_} < 1)) {
-            $this->_error('Expected positive non-zero listen queue size');
-         } elsif (!$this->connectionless) { 
-            $listen = $argv{$_};
-         }
       } else {
          $this->_error("Invalid argument '%s'", $_);
       }
 
-      if (defined($this->[_ERROR])) {
-         return wantarray ? (undef, $this->[_ERROR]) : undef;
+      if (defined($this->{_error})) {
+         return wantarray ? (undef, $this->{_error}) : undef;
       }
 
    }
 
-   # Pack the source address and port information
-   $this->[_SRCADDR] = $this->_addr_pack($src_port, $src_addr);
+   # Pack the socket name (sockaddr) information.
+   $this->{_sock_name} = $this->_name_pack($sock_nh);
 
-   # Pack the destination address and port information
-   $this->[_DSTADDR] = $this->_addr_pack($dst_port, $dst_addr);
+   # Pack the destination name (sockaddr) information.
+   $this->{_dest_name} = $this->_name_pack($dest_nh);
 
    # For all connection-oriented transports and for each unique source 
    # address for connectionless transports, create a new socket. 
 
-   if ((!$this->connectionless) || (!exists($SOCKETS->{$this->[_SRCADDR]}))) {
+   if ((!$this->connectionless) || (!exists($SOCKETS->{$this->{_sock_name}}))) {
 
-      # Create and bind a new IO::Socket::INET[6] object
+      # Create a new IO::Socket object.
 
-      $this->[_SOCKET] = $this->_socket_create(
+      if (!defined($this->{_socket} = $this->_socket_create)) {
+         $this->_perror('Failed to open %s socket', $this->type);
+         return wantarray ? (undef, $this->{_error}) : undef
+      }
 
-         LocalAddr  => $this->_addr_ntoa($src_addr),
-         ($src_port) ? ( LocalPort => $src_port ) : (), 
+      DEBUG_INFO('opened %s socket [%d]', $this->type, $this->fileno);
 
-         # For connection-oriented transports, we either listen or
-         # attempt to connect to the remote host.
+      # Bind the socket.
 
-         (!$this->connectionless) ? ($listen) ?
-            ( Listen   => $listen ) : 
-            ( PeerAddr => $this->_addr_ntoa($dst_addr),
-              PeerPort => $dst_port ) : ()
+      if (!defined($this->{_socket}->bind($this->{_sock_name}))) {
+         $this->_perror('Failed to bind %s socket', $this->type);
+         return wantarray ? (undef, $this->{_error}) : undef
+      }
 
-      );
+      # For connection-oriented transports, we either listen or connect.
 
-      if (!defined($this->[_SOCKET])) {
-         $this->_error($! || 'Unknown error creating socket');
-         return wantarray ? (undef, $this->[_ERROR]) : undef;
+      if (!$this->connectionless) {
+         if ($listen) {
+            if (!defined($this->{_socket}->listen($listen))) {
+               $this->_perror('Failed to listen on %s socket', $this->type);
+               return wantarray ? (undef, $this->{_error}) : undef
+            }
+         } else {
+            if (!defined($this->{_socket}->connect($this->{_dest_name}))) {
+               $this->_perror(
+                  "Failed to connect to remote host '%s'", $this->dest_hostname 
+               );
+               return wantarray ? (undef, $this->{_error}) : undef
+            }
+         }
       }
 
       # Flag the socket as non-blocking outside of socket creation or 
       # the object instantiation fails on some systems (e.g. MSWin32). 
 
-      $this->[_SOCKET]->blocking(FALSE);
-
-      DEBUG_INFO(
-         'opened %s socket [%d]', $this->name, $this->[_SOCKET]->fileno
-      );
+      $this->{_socket}->blocking(FALSE);
 
       # Add the socket to the global socket list with a reference
       # count to track when to close the socket and the maxMsgSize
       # associated with this new object for connectionless transports.
 
       if ($this->connectionless) {
-         $SOCKETS->{$this->[_SRCADDR]} = [ 
-            $this->[_SOCKET], 1, $this->[_MAXSIZE]
+         $SOCKETS->{$this->{_sock_name}} = [ 
+            $this->{_socket}, 1, $this->{_max_msg_size}
          ];
       }
 
    } else {
 
-      DEBUG_INFO(
-         'reused %s socket [%d]',
-         $this->name, $SOCKETS->{$this->[_SRCADDR]}->[_SHARED_SOCKET]->fileno
-      );
+      # Bump up the reference count.
+      $SOCKETS->{$this->{_sock_name}}->[_SHARED_REFC]++;
 
-      # Bump up the reference count
-      $SOCKETS->{$this->[_SRCADDR]}->[_SHARED_REFC]++;
+      # Assign the socket to the object.
+      $this->{_socket} = $SOCKETS->{$this->{_sock_name}}->[_SHARED_SOCKET]; 
 
-      # Adjust the shared maxMsgSize if necessary
-      $this->_shared_maxsize($this->[_MAXSIZE]);
+      # Adjust the shared maxMsgSize if necessary.
+      $this->_shared_max_size($this->{_max_msg_size});
 
-      # Assign the socket to the object
-      $this->[_SOCKET] = $SOCKETS->{$this->[_SRCADDR]}->[_SHARED_SOCKET]; 
+      DEBUG_INFO('reused %s socket [%d]', $this->type, $this->fileno);
 
    }
 
@@ -584,32 +685,54 @@ sub _new
    wantarray ? ($this, '') : $this;
 }
 
-sub _shared_maxsize
+sub _service_resolve
+{
+   my ($this, $serv, $nh) = @_;
+
+   $nh->{port} = undef;
+
+   if ($serv !~ /^\d+$/) {
+      my $port = ($serv =~ s/\((\d+)\)$//) ? ($1 <= 65535) ? $1 : undef : undef;
+      $nh->{port} = getservbyname($serv, $this->_protocol_name) || $port;
+      if (!defined($nh->{port})) {
+         $this->_error(
+            "Unable to resolve %s service name '%s'", $this->type, $_[1]
+         );
+      }
+   } elsif ($serv > 65535) {
+      $this->_error('Invalid %s port number specified [%s]', $this->type, $serv);
+   } else {
+      $nh->{port} = $serv;
+   }
+
+}
+
+sub _shared_max_size
 {
    my ($this, $size) = @_;
 
    # Connection-oriented transports do not share sockets.
    if (!$this->connectionless) {
-      return $this->[_MAXSIZE];
+      return $this->{_max_msg_size};
    }
 
    if (@_ == 2) {
 
       # Handle calls during object creation.
-      if (!defined($this->[_SRCADDR])) {
-         return $this->[_MAXSIZE];
+      if (!defined($this->{_sock_name})) {
+         return $this->{_max_msg_size};
       }
 
       # Update the shared maxMsgSize if the passed
       # value is greater than the current size.
 
-      if ($size > $SOCKETS->{$this->[_SRCADDR]}->[_SHARED_MAXSIZE]) {
-         $SOCKETS->{$this->[_SRCADDR]}->[_SHARED_MAXSIZE] = $size;
+      if ($size > $SOCKETS->{$this->{_sock_name}}->[_SHARED_MAXSIZE]) {
+         $SOCKETS->{$this->{_sock_name}}->[_SHARED_MAXSIZE] = $size;
       }
 
    }
 
-   $SOCKETS->{$this->[_SRCADDR]}->[_SHARED_MAXSIZE];
+   $SOCKETS->{$this->{_sock_name}}->[_SHARED_MAXSIZE];
 }
 
 sub _msg_size_default
@@ -617,21 +740,44 @@ sub _msg_size_default
    MSG_SIZE_DEFAULT;
 }
 
-sub _reasm_reset
-{
-   $_[0]->[_REASM_BUFFER] = '';
-   $_[0]->[_REASM_LENGTH] = 0;
-}
-
 sub _error
 {
    my $this = shift;
 
-   if (!defined($this->[_ERROR])) {
-      $this->[_ERROR] = (@_ > 1) ? sprintf(shift(@_), @_) : $_[0];
+   if (!defined($this->{_error})) {
+      $this->{_error} = (@_ > 1) ? sprintf(shift(@_), @_) : $_[0];
       if ($this->debug) {
          printf("error: [%d] %s(): %s\n",
-            (caller(0))[2], (caller(1))[3], $this->[_ERROR]
+            (caller(0))[2], (caller(1))[3], $this->{_error}
+         );
+      }
+   }
+
+   return;
+}
+
+sub strerror()
+{
+   if ($! =~ /^Unknown error/) {
+      return sprintf('%s', $^E) if ($^E);
+      require Errno;
+      map { return sprintf('Error %s', $_) if ($!{$_}); } keys(%!);
+      return sprintf('%s (%d)', $!, $!);
+   }
+
+   $! ? sprintf('%s', $!) : 'No error';
+}
+
+sub _perror
+{
+   my $this = shift;
+
+   if (!defined($this->{_error})) {
+      $this->{_error}  = ((@_ > 1) ? sprintf(shift(@_), @_) : $_[0]) || '';
+      $this->{_error} .= ($this->{_error}) ? ': ' . strerror() : strerror();
+      if ($this->debug) {
+         printf("error: [%d] %s(): %s\n",
+            (caller(0))[2], (caller(1))[3], $this->{_error}
          );
       }
    }
@@ -642,7 +788,7 @@ sub _error
 sub _error_clear
 {
    $! = 0;
-   $_[0]->[_ERROR] = undef;
+   $_[0]->{_error} = undef;
 }
 
 {
